<?php

namespace Drupal\session_reminder\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\session_reminder\Service\SessionReminderService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a ConfigForm to manage Session Reminder settings.
 */
class ConfigForm extends ConfigFormBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The session reminder service.
   *
   * @var \Drupal\session_reminder\Service\SessionReminderService
   */
  protected SessionReminderService $sessionReminderService;

  /**
   * Constructs a ConfigForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\session_reminder\Service\SessionReminderService $session_reminder_service
   *   The session reminder service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, SessionReminderService $session_reminder_service) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entity_type_manager;
    $this->sessionReminderService = $session_reminder_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('session_reminder.session_reminder_service'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'sr__config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('session_reminder.settings');
    $cookieLifetime = $this->sessionReminderService->getCookieLifetime();
    $expTimestamp = $this->sessionReminderService->getExpTimestamp();
    if (!$cookieLifetime) {
      $this->messenger()->addError($this->t('Unable to detect Cookie Lifetime or set to 0 (no expire).'));
      $this->messenger()->addError($this->t('You need to set a value for session.cooke_lifetime in services.yml to enabled alert.'));
      return $form;
    }

    $form['sr__config_form'] = [
      '#type' => 'details',
      '#title' => $this->t('Settings'),
      '#open' => TRUE,
    ];

    $expDate = ($expTimestamp > 0) ? DrupalDateTime::createFromTimestamp($expTimestamp) : NULL;
    $expDateFormatted = ($expDate) ? $expDate->format('Y-m-d H:i:s') : "N/A";
    $seconds = $config->get('warning_threshold');
    $modalTimestamp = ($expTimestamp > 0) ? ($expTimestamp - $seconds) : NULL;
    $modalDate = ($expTimestamp > 0 && $modalTimestamp > 0) ? DrupalDateTime::createFromTimestamp($modalTimestamp) : NULL;
    $modalDateFormatted = ($modalDate) ? $modalDate->format('Y-m-d H:i:s') : "N/A";
    $description = [
      $this->t("Your Drupal's Cookie Session are set to expire after @lifetime seconds.", ['@lifetime' => $cookieLifetime]),
      $this->t('Your Current Cookie Session will expire on @date.', ['@date' => $expDateFormatted]),
      $this->t('Modal for your session will be shown on @datemodal.', ['@datemodal' => $modalDateFormatted]),
    ];

    // Make array of translatable descriptions.
    $descriptionMarkup = Markup::create($this->t('<b>Details</b>: <br> - @item1 <br> - @item2 <br> - @item3', [
      '@item1' => $description[0],
      '@item2' => $description[1],
      '@item3' => $description[2],
    ]));

    $form['sr__config_form']['description'] = [
      '#type' => 'container',
      '#markup' => $descriptionMarkup,
      '#attributes' => ['style' => 'font-size:10pt; margin-bottom: 10px'],
      '#open' => TRUE,
    ];

    $form['sr__config_form']['warning_threshold'] = [
      '#type' => 'number',
      '#description' => $this->t('Set how many seconds before the session expiration the modal should appear.'),
      '#title' => $this->t('Warning Threshold'),
      '#max' => $cookieLifetime,
      '#min' => 10,
      '#required' => TRUE,
      '#default_value' => $config->get('warning_threshold'),
    ];

    $options = $this->getRoles();

    $form['sr__config_form']['roles'] = [
      '#type' => 'select',
      '#title' => $this->t('Roles'),
      '#description' => $this->t('Select one or more roles that will see the modal'),
      '#options' => $options,
      '#multiple' => TRUE,
      '#required' => TRUE,
      '#default_value' => $config->get('roles') ?: ['authenticated'],
    ];

    $form['sr__config_form_style'] = [
      '#type' => 'details',
      '#title' => $this->t('Style'),
      '#open' => TRUE,
    ];

    $template = "
        <div id='preview--sr--modal-form'>
          <span class='preview--sr--modal-form--cancel'>X</span>
          <div>
            <span id='preview--sr--modal-form--h2'>" . $this->t('Session expiring') . "</span>
            <span id='preview--sr--modal-form--p'>" . $this->t("Your session will expire in @sec seconds", ['@sec' => 0]) . "</span>
          </div>
          <div class='preview--sr--form-actions'>
            <div id='preview--sr--modal-form--btn'>" . $this->t('Extend Session') . "</div>
          </div>
        </div>";

    $form['sr__config_form_style']['live_preview_description'] = [
      '#type' => 'container',
      '#markup' => "<p><b>" . $this->t('Live Preview') . "</b></p>",
    ];

    $form['sr__config_form_style']['live_preview'] = [
      '#type' => 'container',
      '#markup' => $template,
      '#attributes' => ['id' => 'preview--sr--modal-form-container'],
    ];

    $form['sr__config_form_style']['fields'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['horizontal-form']],
    ];

    $form['sr__config_form_style']['fields']['color_modal_bg'] = [
      '#type' => 'color',
      '#title' => $this->t('Modal Background Color'),
      '#default_value' => $config->get('color_modal_bg') ?? '#ffffff',
      '#description' => $this->t('Choose the color for the Modal background.'),
      '#attributes' => [
        'class' => ['live-preview', 'inline-field'],
        'data-preview-target' => 'preview--sr--modal-form',
        'data-preview-property' => 'background-color',
      ],
      '#required' => TRUE,
    ];

    $form['sr__config_form_style']['fields']['color_modal_title'] = [
      '#type' => 'color',
      '#title' => $this->t('Modal Title Color'),
      '#default_value' => $config->get('color_modal_title') ?? '#ffffff',
      '#description' => $this->t('Choose the color for the Modal title.'),
      '#attributes' => [
        'class' => ['live-preview', 'inline-field'],
        'data-preview-target' => 'preview--sr--modal-form--h2',
        'data-preview-property' => 'color',
      ],
      '#required' => TRUE,
    ];

    $form['sr__config_form_style']['fields']['color_modal_text'] = [
      '#type' => 'color',
      '#title' => $this->t('Modal Text Color'),
      '#default_value' => $config->get('color_modal_text') ?? '#ffffff',
      '#description' => $this->t('Choose the color for the Modal text.'),
      '#attributes' => [
        'class' => ['live-preview', 'inline-field'],
        'data-preview-target' => 'preview--sr--modal-form--p',
        'data-preview-property' => 'color',
      ],
      '#required' => TRUE,
    ];

    $form['sr__config_form_style']['fields']['color_btn_bg'] = [
      '#type' => 'color',
      '#title' => $this->t('Submit Button Bg Color'),
      '#default_value' => $config->get('color_btn_bg') ?? '#20478c',
      '#description' => $this->t("Choose the color for the 'Extend Session'/'Close' button."),
      '#attributes' => [
        'class' => ['live-preview', 'inline-field'],
        'data-preview-target' => 'preview--sr--modal-form--btn',
        'data-preview-property' => 'background-color',
      ],
      '#required' => TRUE,
    ];

    $form['sr__config_form_style']['fields']['color_btn_text'] = [
      '#type' => 'color',
      '#title' => $this->t('Submit Button Text Color'),
      '#default_value' => $config->get('color_btn_text') ?? '#ffffff',
      '#description' => $this->t("Choose the color for the 'Extend Session'/'Close' text.'"),
      '#attributes' => [
        'class' => ['live-preview', 'inline-field'],
        'data-preview-target' => 'preview--sr--modal-form--btn',
        'data-preview-property' => 'color',
      ],
      '#required' => TRUE,
    ];

    $form['sr__config_form_style']['fields']['modal_border_radius'] = [
      '#type' => 'number',
      '#title' => $this->t('Modal Border Radius'),
      '#default_value' => $config->get('modal_border_radius') ?? 10,
      '#description' => $this->t('Choose the border radius form modal.'),
      '#max' => 999,
      '#min' => 0,
      '#attributes' => [
        'class' => ['live-preview', 'inline-field'],
        'data-preview-target' => 'preview--sr--modal-form',
        'data-preview-property' => 'border-radius',
      ],
      '#required' => TRUE,
    ];

    $form['#attached']['drupalSettings']['session_reminder_lp']['session_expire_timestamp'] = $this->sessionReminderService->getExpTimestamp();
    $form['#attached']['library'][] = 'session_reminder/modal-form-preview-library';
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('session_reminder.settings')
      ->set('warning_threshold', $form_state->getValue('warning_threshold'))
      ->set('roles', $form_state->getValue('roles'))
      ->set('color_btn_bg', $form_state->getValue('color_btn_bg'))
      ->set('color_btn_text', $form_state->getValue('color_btn_text'))
      ->set('color_modal_title', $form_state->getValue('color_modal_title'))
      ->set('color_modal_bg', $form_state->getValue('color_modal_bg'))
      ->set('color_modal_text', $form_state->getValue('color_modal_text'))
      ->set('modal_border_radius', $form_state->getValue('modal_border_radius'))
      ->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Get Roles array without anonymous.
   *
   * @return array
   *   Roles in format[ID:label]
   */
  private function getRoles(): array {
    try {
      $role_storage = $this->entityTypeManager->getStorage('user_role');
    }
    catch (\Exception $e) {
      $this->logger("session_reminder")->error("Error: " . $e->getMessage());
      return [];
    }

    $roles = $role_storage->loadMultiple();
    $results = [];
    foreach ($roles as $role_id => $role) {
      if ($role->id() == AccountInterface::ANONYMOUS_ROLE) {
        continue;
      }
      $results[$role_id] = $role->label();
    }

    return $results;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['session_reminder.settings'];
  }

}
