<?php

namespace Drupal\session_reminder\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\session_reminder\Service\SessionReminderService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Provides a Form for a modal form with a refresh button and countdown.
 */
class ModalForm extends FormBase {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The session reminder service.
   *
   * @var \Drupal\session_reminder\Service\SessionReminderService
   */
  protected SessionReminderService $sessionReminderService;

  /**
   * Constructs a new ModalForm object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\session_reminder\Service\SessionReminderService $session_reminder_service
   *   The session reminder service.
   */
  public function __construct(RequestStack $request_stack, ConfigFactoryInterface $config_factory, SessionReminderService $session_reminder_service) {
    $this->requestStack = $request_stack;
    $this->configFactory = $config_factory;
    $this->sessionReminderService = $session_reminder_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('request_stack'),
      $container->get('config.factory'),
      $container->get('session_reminder.session_reminder_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'sr__modal_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    if (!$this->sessionReminderService->userEnabled()) {
      return $form;
    }

    $config = $this->configFactory->get('session_reminder.settings');
    $colorBtnBg = $config->get('color_btn_bg');
    $colorBtnText = $config->get('color_btn_text');
    $colorModalTitle = $config->get('color_modal_title');
    $colorModalBg = $config->get('color_modal_bg');
    $colorModalText = $config->get('color_modal_text');
    $modalBorderRadius = $config->get('modal_border_radius');

    $form['sr__modal_form']['cancel'] = [
      '#type' => 'markup',
      '#markup' => '<a class="sr--modal-form--cancel">X</a>',
    ];

    $form['sr__modal_form']['title'] = [
      '#type' => 'container',
      '#markup' => '<h2 class="sr--modal-form--title">' . $this->t('Session expiring') . '</h2>',
      '#attributes' => [
        'style' => "color: $colorModalTitle;",
      ],
    ];

    $form['sr__modal_form']['description'] = [
      '#type' => 'container',
      '#markup' => '<p class="sr--modal-form--description">' . $this->t("Your session will expire in @sec seconds", ['@sec' => 0]) . '</p>',
      '#attributes' => [
        'style' => "color: $colorModalText;",
      ],
    ];

    $form['sr__modal_form']['actions'] = ['#type' => 'actions'];

    $form['sr__modal_form']['actions']['submit'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => $this->t('Extend Session'),
      '#attributes' => [
        'class' => ['sr--modal-form--refresh-btn'],
        'style' => "background-color: $colorBtnBg; color: $colorBtnText;",
      ],
    ];

    $form['sr__modal_form']['actions']['close'] = [
      '#type' => 'button',
      '#button_type' => 'primary',
      '#value' => $this->t('Close'),
      '#attributes' => [
        'class' => ['sr--modal-form--close-btn'],
        'style' => "background-color: $colorBtnBg; color: $colorBtnText;",
      ],
      '#submit' => ['::reloadPage'],
    ];

    $style = 'background-color: ' . $colorModalBg . ';';
    $style .= 'border-radius: ' . $modalBorderRadius . 'px;';
    $form['#attributes']['id'][] = 'sr--modal-form';
    $form['#attributes']['class'][] = 'sr--modal-form';
    $form['#attributes']['style'] = $style;
    $form['#cache'] = ['max-age' => 0];
    $form['#attached']['library'][] = 'session_reminder/modal-form-library';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $request = $this->requestStack->getCurrentRequest();
    /** @var \Symfony\Component\HttpFoundation\Session\SessionInterface $session */
    $sessionCookieName = $request->getSession()->getName();

    if ($this->sessionReminderService->userEnabled() && !empty($sessionCookieName) && $request->cookies->get($sessionCookieName)) {
      $sessionCookieValue = $request->cookies->get($sessionCookieName);
      $addSeconds = ini_get('session.cookie_lifetime');
      $newExpiringDatetime = time() + $addSeconds;
      setcookie(
        $sessionCookieName,
        $sessionCookieValue,
        $newExpiringDatetime,
        ini_get('session.cookie_path'),
        ini_get('session.cookie_domain'),
        ini_get('session.cookie_httponly'),
        ini_get('session.cookie_secure')
      );
      $this->sessionReminderService->setExpTimestamp($newExpiringDatetime);
    }
  }

  /**
   * Manage the reloadPage, used in Close Button.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return void
   *   Redirect
   */
  public function reloadPage(array &$form, FormStateInterface $form_state): void {
    // Refresh current page.
    $form_state->setRedirect('<current>');
  }

}
