<?php

namespace Drupal\session_reminder\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\HttpFoundation\Session\SessionInterface;

/**
 * Service for handling session reminders.
 */
class SessionReminderService {
  const SESSION_EXP_DATETIME = "session_reminder__exp_timestamp";

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected AccountProxyInterface $currentUser;

  /**
   * The session service.
   *
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected SessionInterface $session;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $config;

  /**
   * Constructs a SessionReminderService object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Symfony\Component\HttpFoundation\Session\SessionInterface $session
   *   The session service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(AccountProxyInterface $current_user, SessionInterface $session, ConfigFactoryInterface $config_factory) {
    $this->currentUser = $current_user;
    $this->session = $session;
    $this->config = $config_factory;
  }

  /**
   * Get Session Expiring Datetime from the session.
   *
   * @return int|null
   *   Session Expiring Datetime from the session.
   */
  public function getExpTimestamp(): int|null {
    if ($this->session->has(self::SESSION_EXP_DATETIME)) {
      return $this->session->get(self::SESSION_EXP_DATETIME);
    }
    else {
      // Get last accessed time.
      if ($this->currentUser->getLastAccessedTime()) {
        return $this->currentUser->getLastAccessedTime() + self::getCookieLifetime();
      }
    }

    return NULL;
  }

  /**
   * Set Session Expiring Datetime in session.
   *
   * @param int|null $timestamp
   *   Timestamp.
   *
   * @return void
   *   No return.
   */
  public function setExpTimestamp(int|null $timestamp = NULL): void {
    /** @var \Symfony\Component\HttpFoundation\Session\SessionInterface $session */
    $this->session->set(self::SESSION_EXP_DATETIME, $timestamp);
  }

  /**
   * Check if the user can see the modal.
   *
   * @return bool
   *   User can show the modal
   */
  public function userEnabled(): bool {
    $authenticated = $this->currentUser->isAuthenticated();
    $expTimestamp = self::getExpTimestamp();

    if ($authenticated && !empty($expTimestamp)) {
      // If the session timestamp is not set, not load the modal.
      $roles = array_flip($this->currentUser->getRoles());
      $enabledRoles = $this->config->get('session_reminder.settings')->get('roles');

      return (count(array_intersect_key($roles, $enabledRoles)) >= 1);
    }

    return FALSE;
  }

  /**
   * Get Cookie Lifetime defined in services.yml.
   */
  public static function getCookieLifetime(): false|string {
    return ini_get("session.cookie_lifetime");
  }

}
